/*
 * win/diagram.h
 */

#ifndef _WIN_DIAGRAM_H
#define _WIN_DIAGRAM_H

#include <stdio.h>
#include <alib/iffrw.h>

// WDiaSlot flags
#define WDSF_VERTICAL   1       // Slot flags; vertical slot (otherwise Hor)

// WDiagram flags[] (for blocks)
#define WDF_HILITE      1       // Hilite box
#define WDF_WARN        2       // Notice that this box needs attention

typedef struct WDiaSlot
// Minimal slot info for WDiagram slot array
{ short x,y;
  char  len,flags;
  short n;                  // Slot index
} WDiaSlot;

typedef struct WDiagram
// Don't change this struct lightly without checking source dependencies
// (sizeof(short), WinDiaLoad()/Save()/IFF versions)
{ short wid,                            /* Dimension used (blocks) */
        hgt;
  short bwid,bhgt;                      /* Single block dimensions */
  short bx,by;                          /* Actual block plot first coor */
  short dx,dy,                          /* Paint destination preference */
        dwid,dhgt;                      /* Preferred wid/hgt (max!) */
  short slots;
  //char  state[MAXCOL][MAXROW];          /* unused=2 black=1 white=0 */
  char  *flags;                         /* Any flags */
  char  *letter;                        /* Current letter fill */
  short *number;                        /* Any small number? */
  WDiaSlot *slot;                       // Slot definitions
  //char  number_ll[MAXCOL][MAXROW];      /* Any small number lower-left? */
} WDiagram;

// diagram.c
WDiagram *WinDiaCreate(void);

// Diagram definition
bool WinDiaSetSize(WDiagram *dia,int wid,int hgt);
bool WinDiaSetSlots(WDiagram *dia,int slots);

char WinDiaGetLetter(WDiagram *dia,int x,int y);
void WinDiaSetLetter(WDiagram *dia,int x,int y,char letter);
char WinDiaGetFlags(WDiagram *dia,int x,int y);
void WinDiaSetFlags(WDiagram *dia,int x,int y,char flags);
short WinDiaGetNumber(WDiagram *dia,int x,int y);
void  WinDiaSetNumber(WDiagram *dia,int x,int y,short number);

WDiaSlot *WinDiaGetSlot(WDiagram *dia,int n);

void WinDiaCalcBlockSize(WDiagram *dia);
void WinDiaCheckLMB(WDiagram *dia);
void WinDiaCheckRMB(WDiagram *dia);
void WinDiaPaintBlockFlags(WDiagram *dia,short x,short y,short flags);
void WinDiaPaintBlock(WDiagram *dia,short x,short y);
void WinDiaPaint(WDiagram *dia);
short WinDiaIsFilledLetter(WDiagram *dia);
short WinDiaIsFilledNumber(WDiagram *dia);
void WinDiaFindRight(WDiagram *dia,short x,short y,short *dx,short *dy);
void WinDiaFindLeft(WDiagram *dia,short x,short y,short *dx,short *dy);
short WinDiaHasLetters(WDiagram *dia);
void WinDiaNew(WDiagram *dia);
int  WinDiaCountLetterSpots(WDiagram *dia);

// I/O
bool WinDiaLoad(WDiagram *dia,FILE *fp);
bool WinDiaLoadIFF(WDiagram *dia,struct IFFR *ir);
bool WinDiaSave(WDiagram *dia,FILE *fp);
bool WinDiaSaveIFF(WDiagram *dia,struct IFFW *iw);

void WinDiaDestroy(WDiagram *dia);

// Slots (WDiaSlot)
void WinDiaCalcNextXY(WDiagram *dia,int *px,int *py,int flags);
WDiaSlot *WinDiaFindSlotAt(WDiagram *dia,int mx,int my,int dir);
bool WinDiaSlotCrosses(WDiagram *dia,WDiaSlot *s,int x,int y);
void WinDiaSlotHilite(WDiagram *dia,WDiaSlot *s,bool yn);
void WinDiaSlotWarn(WDiagram *dia,WDiaSlot *s,bool yn);
bool WinDiaSlotIsWarn(WDiagram *dia,WDiaSlot *s);
void WinDiaSlotPaint(WDiagram *dia,WDiaSlot *s);

#endif
