/* solvtask.h */
// 28-8-97: Added proverb string to SolverJob
// 23-4-99: SolverTask file may be in multiple locations (network/local)

#ifndef __SOLVTASK_H
#define __SOLVTASK_H

#include <alib/standard.h>
#include <alib/iff.h>
#include <win/standard.h>

//#define ST_NAME         "SOLVER.TSK"
#define ST_NAME         SolverTaskGetFile()
#define SOLVERTASK_NAME SolverTaskGetFile()
#define SOLVERTASK_VERSION  1       /* File format version */
#define SJ_MAXOUTNAME       16      /* Max output file chars */

#define ST_LOCAL        0           // Solver task file location
#define ST_NETWORK      1

#define ID_TASK         IFF_ID('T','A','S','K')
#define ID_TKHD         IFF_ID('T','K','H','D')
#define ID_JOB1         IFF_ID('J','O','B','1')
// Since end of august 1997, extra proverb information
#define ID_JOB2         IFF_ID('J','O','B','2')

#define MAXPRESLOT      15
#define MAXPRESLOTWLEN  (FS_WOORD+1)    /* Word length for preslot */

#define PUZTYPE_KRUIS   0               /* Possible puztypes */
#define PUZTYPE_LEG     1               /* Legraadsel */
#define PUZTYPE_CRYPT1  2
#define PUZTYPE_CRYPT2  3
#define PUZTYPE_ZWEEDS  4
#define PUZTYPE_DOOR    5               // Dreaded doorlopers
#define PUZTYPE_ANNA    6               // Anagrammen
#define PUZTYPE_QUIZ    7               // 6-8-97
#define PUZTYPE_MAX     7

#define OUTTYPE_ONDER   0               /* Output types */
#define OUTTYPE_ACHTER  1
#define OUTTYPE_ZETWERK 2
#define OUTTYPE_ZWEEDS  3
#define OUTTYPE_DOOR    4
#define OUTTYPE_MAX     4

#define FEF_SOLUTION    1               /* File entry flags: output solu too */
#define FEF_TABDIAG     2               // TABs inbetween solu chars
                                        // (use only in conj. with OUTPUTDIAG)
#define FEF_SPLITOUT    4               // Split output file into 3 files
#define FEF_PROCESS     8               // Update usage dates when solved?
#define FEF_OUTPUTDIAG  0x10            // Output diagram (may use FEF_TABDIAG)
#define FEF_LOWERDIAG   0x20            // Use lowercast for diagram letters
#define FEF_DBLPROVLETS 0x40            // Allow doubly used proverb letters

#define EC_SOLVED       0               /* Exit codes for external programs */
#define EC_NOTSOLVED    1
#define EC_ABORTPROGRAM 2

#define STHF_OVERDRIVE  1               // Fast loading & solving

struct SolverTaskHeader
{ char version,
       outdir[15];                  /* Output directory (C:\OUTPUT) */
  long cluster_id;                  /* This task's cluster number */
  long flags;                       /* STHF_xxx */
};
// When changing struct SolverJob, please recompile (entirely):
// INVOERN, SOLVE_F, SOLVE_X, SOLVE_O, SOLVER
struct SolverJob                    /* Was a FileEntry */
{ char  solved,                     /* Puzzle is solved? (1=YES) */
        diagname[9],                /* Filename.PUZ */
        puztype,                    /* PUZTYPE_xxx (+10?) */
        outtype;                    /* OUTTYPE_xxx */
  char  outname[SJ_MAXOUTNAME];
  int16 maxtime,                    /* Time to solve (mins) */
        age,                        /* Age of words */
        age4,                       /* Special len==4 age */
        age5,                       /* Special len==5 age */
        age6,                       // Special 6lw age (15-8-96)
        mindifdesc,                 /* Min. difficulty of descriptions */
        maxdifdesc,
        mindifword,                 /* Minimal difficulty of words */
        maxdifword,
        attempts,                   /* #attempts */
        dt_level;                   /* DeepThought level */
  char  pres,                       /* #prefilled slots */
        flags;                      /* FEF_xxx */
  int   pre_n[MAXPRESLOT];          /* Number of slot in puzzle terms */
  char  pre_or[MAXPRESLOT],         /* Orientation */
        pre_s[MAXPRESLOT][MAXPRESLOTWLEN]; /* The words */
  char  wtype_excl[10];             /* WOORDTYPE field exclude chars */
  char  proverb[250];               // Citaat max length (proverb=100) JOB2
};
struct SolverJob96                  // Old (<28-8-97) job
{ char  solved,                     /* Puzzle is solved? (1=YES) */
        diagname[9],                /* Filename.PUZ */
        puztype,                    /* PUZTYPE_xxx (+10?) */
        outtype;                    /* OUTTYPE_xxx */
  char  outname[SJ_MAXOUTNAME];
  int16 maxtime,                    /* Time to solve (mins) */
        age,                        /* Age of words */
        age4,                       /* Special len==4 age */
        age5,                       /* Special len==5 age */
        age6,                       // Special 6lw age (15-8-96)
        mindifdesc,                 /* Min. difficulty of descriptions */
        maxdifdesc,
        mindifword,                 /* Minimal difficulty of words */
        maxdifword,
        attempts,                   /* #attempts */
        dt_level;                   /* DeepThought level */
  char  pres,                       /* #prefilled slots */
        flags;                      /* FEF_xxx */
  int   pre_n[MAXPRESLOT];          /* Number of slot in puzzle terms */
  char  pre_or[MAXPRESLOT],         /* Orientation */
        pre_s[MAXPRESLOT][MAXPRESLOTWLEN]; /* The words */
  //char  pre_n[MAXPRESLOT],          /* Number of slot in puzzle terms */
  //      pre_or[MAXPRESLOT],         /* Orientation */
  //      pre_s[MAXPRESLOT][MAXPRESLOTWLEN]; /* The words */
  char  wtype_excl[10];             /* WOORDTYPE field exclude chars */
};

void SolverTaskSetFile(int where);
string SolverTaskGetFile(void);

bool SolverTaskJobRead(char *fname,struct SolverTaskHeader *sth,
  struct SolverJob *sj,int n);
bool SolverTaskSolved(char *fname,int n);       // Set to Solved state
bool SolverTaskIsSolved(char *fname,int n);     // Ask Solved state

char SolverJobOmsType(struct SolverJob *sj);
char SolverJobOmsTable(struct SolverJob *sj);

#endif

