/** win/standard.h - default known values and constants (records too)
    Prototypes of WinLib modules too.
**/

#ifndef __WIN_STANDARD_H
#define __WIN_STANDARD_H

#include <alib/standard.h>
#include <alib/types.h>
#include <d4all.h>

/** Maximum puzzle diagram dimensions **/
#define MAX_PUZWID      250             // A puzzle as big as a wall!
#define MAX_PUZHGT      250             // For a char[][] this is just <64K

/** Orientation values **/
#define OR_HOR          0
#define OR_VERT         1

/** Very constant field sizes of WOORDEN table **/
#define FS_WOORD        30
#define FS_WOORDTYPE    10

/** Somewhat constant field sizes of OMS_x tables **/
#define FS_OMS          100
#define FS_EIGENSCH     10
#define FS_OFLAGS       5

/** Constant field sizes of W_THEMA.DBF **/
#define FS_THEMA        40

/** CITATEN.DBF **/
#define FS_CITAAT       250
#define FS_GECITEERDE   40

/** GEZEGDEN.DBF **/
#define FS_PROVERB      100
#define FS_OCC          10

// Graad recording (FLAG_x in WOORDEN.DBF)
#define WIN_MIN_FLAG_GRAAD  1
#define WIN_MAX_FLAG_GRAAD  9

// Typically used WP4.2 chars
#define WP42_TAB            9
#define WP42_BOLD_ON        0x9d
#define WP42_BOLD_OFF       0x9c

typedef struct WinRecOms
// A record of the OMS_x.DBF tables
{ long recnr;
  char woord[FS_WOORD+1];
  char oms[FS_OMS+1];
  int  graad;
  char flags[FS_OFLAGS+1];
  char eigensch[FS_EIGENSCH+1],
       pad;
} WinRecOms;
typedef struct WinRecWoord
// A record of WOORDEN.dbf (partially)
{ long recnr;
  char woord[FS_WOORD+1],
       graad;
  int  clusterID;
  char d_creation[8+1],
       woordType[FS_WOORDTYPE+1],
       alt[FS_WOORD+1],
       woord_wr[FS_WOORD+1],
       alt_wr[FS_WOORD+1];
} WinRecWoord;

// words.c
string wlMakeKey(cstring w,int len);
void wlStripSpaces(string s);
bool wlWordIsValid(cstring s);
void WinOmsAddLen(string oms,int wlen);
// Database ops
bool wlWoordGetRec(cstring w,WinRecWoord *r);
bool wlWoordPutRec(cstring w,WinRecWoord *r);
long WinFindOms(string w,char *dbuf,int maxograad,char oms_type,DATA4 *data);
//bool WinFindWriting(char *w,char *d,DATA4 *data_w);
bool wlFindWriting(cstring w,string d,DATA4 *data_w);
bool WinWordAdd(string w);
long WinWordRecnr(string w,DATA4 *dataW);
bool WinWordExists(string w,DATA4 *dataW);      // def. dataW=0
bool WinUpdateModification(string w,char omsType,DATA4 *dataW); // [dataW=0]

// oms.c
typedef bool (*CBFORALLOMS)(WinRecOms *wro);
bool WinOmsGetRec(char omsType,long recnr,WinRecOms *r);
bool wlForAllOms(cstring w,char oms_type,CBFORALLOMS cbfunc);

// dlgword.c
bool wlEditWord(cstring wrd,cstring sTitle,cstring sOk,cstring sCancel);

// output.c
string WinGetOutputDir(void);                   // Default output location
string WinGetTempDir(void);                     // Default temp location
string StrToWP42(string src,bool containsBold); // Accent conversions
void   StrCapitalize(string s);
//void   WinStripCryptoLength(string s);          // Strip (<len>) stuff.
void   wlStripCryptoLength(string s);          // Strip (<len>) stuff.

// qsearch.c - integrated into WinLib since 6-3-00
bool wlQSearch(string pat,cstring dir);

// solver.c - Filenames for SOLVER
string SolverGetInfoFile(void);
string SolverGetSolutionFile(void);
string SolverGetResourceFile(void);

#endif

