/* wim.h */

#ifndef __WIN_WIM_H
#define __WIN_WIM_H

#ifdef __cplusplus
extern "C" {
#endif

#include <win/standard.h>
#include <alib/standard.h>
#include <alib/iffrw.h>
#include <win/diagram.h>

#define ID_WIM1 IFF_ID('W','I','M','1')
#define ID_WMHD IFF_ID('W','M','H','D')
#define ID_WROM IFF_ID('W','R','O','M')
#define ID_WTXT IFF_ID('W','T','X','T')
#define ID_WDIA IFF_ID('W','D','I','A')

// state & chunksDone masks (state is really not a mask)
#define WIMF_IDLE   0
#define WIMF_WROM   1
#define WIMF_WTXT   2

#define WIMTYPE_SOLVER      0           // Solver file
#define WIMTYPE_FLIPPER     1           // Flipper file

typedef struct WimWordRec
// Record used to store word/oms info
{
  char woord[FS_WOORD+1];
  char oms[FS_OMS+1],
       omsType,pad;
  int  graad;
} WimWordRec;
typedef struct WimHeader
{
  int type;
  int flags;
} WimHeader;
typedef struct Wim
{
  struct IFFW *iw;
  struct IFFR *ir;
  WimHeader hdr;                // File header information
  WDiagram *dia;                // A diagram
  WimWordRec *wrec;             // The words+info
  int wrecs;                    // #words (=#slots?)
  char *text;                   // The original text part (with $...$ refs)
  int state;
  int chunksDone;
} Wim;

typedef struct WimIMRec
{ char woord[FS_WOORD+1],
       oms[FS_OMS+1],
       d_creation[8+1],
       person[10+1],
       omsType,pad;
  long recnr;                   // What recnr on disk?
} WimIMRec;

// wim.c
struct Wim *WIMCreate(string orgName);
void WIMWriteHeader(struct Wim *wim,WimHeader *hdr);
void WIMWriteWord(struct Wim *w,string wrd,string oms,int graad,char omsType);
void WIMprintf(struct Wim *w,char *fmt,...);
void WIMSaveDiagram(struct Wim *w,WDiagram *dia);
void WIMClose(struct Wim *w);
bool WIMSave(Wim *wim,string orgName);          // Save WIM file itself

struct Wim *WIMOpen(string orgName);

void WIMBrotherName(const string orgName,string d);

// wimout.c
bool WIMOutputOriginal(Wim *w,string fname);

// wimdbf.c
typedef bool (*WDCBFunc)(WimIMRec *rec);
bool WIMDbfFind(string woord,string oms);
bool WIMDbfAdd(string woord,string oms,char omsType);
bool WIMDbfIterate(WDCBFunc cbFunc);
bool WIMDbfWipe(void);

#ifdef __cplusplus
}
#endif

#endif

