#ifndef __X11_WIN32_H
#define __X11_WIN32_H

#include <qlib/types.h>

typedef int XID;
typedef bool Bool;

typedef void *Drawable;
typedef void *Window;
typedef int Pixel;
typedef int Status;
typedef int VisualID;
typedef int Time;
typedef XID Font;
typedef XID GC;
typedef XID Cursor;
typedef XID Colormap;
struct _XDisplay
{
};
typedef struct _XDisplay Display;

struct XColor
{
  int pixel;
  int red,green,blue;
  int flags;
};

typedef struct
{
} Visual;

typedef struct
{
  Visual *visual;
  VisualID visualid;
  int screen;
  unsigned int depth;
  int c_class;
  unsigned long red_mask;
  unsigned long green_mask;
  unsigned long blue_mask;
  int colormap_size;
  int bits_per_rgb;
} XVisualInfo;

struct XStandardColormap
{
};

struct XFontStruct
{
};

// Events (taken from X11)
typedef struct
{ int type;
  unsigned long serial;
  Bool send_event;
  Display *dsp;
  Window window;
} XAnyEvent;
typedef struct
{ int type;
  unsigned long serial;
  Bool send_event;
  Display *dsp;
  Window window;
  Window root;
  Window subwindow;
  Time time;
  int x,y;
  int x_root,y_root;
  unsigned int state;
  char is_hint;
  Bool same_screen;
} XMotionEvent;
typedef struct
{ int type;
  unsigned long serial;
  Bool send_event;
  Display *dsp;
  Window window;
  Window root;
  Window subwindow;
  Time time;
  int x,y;
  int x_root,y_root;
  unsigned int state;
  unsigned int keycode;
  Bool same_screen;
} XKeyEvent;
typedef struct
{ int type;
  unsigned long serial;
  Bool send_event;
  Display *dsp;
  Window window;
  Window root;
  Window subwindow;
  Time time;
  int x,y;
  int x_root,y_root;
  unsigned int state;
  unsigned int button;
  Bool same_screen;
} XButtonEvent;
typedef struct
{ int type;
  unsigned long serial;
  Bool send_event;
  Display *dsp;
  Window event;
  Window window;
  int x,y;
  int width,height;
  int border_width;
  Window above;
  Bool override_redirect;
} XConfigureEvent;
typedef struct
{ int type;
  unsigned long serial;
  Bool send_event;
  Display *dsp;
  Window window;
  int x,y;
  int width,height;
  int count;
} XExposeEvent;

typedef union _XEvent
{
   int type;
   XAnyEvent xany;
   XMotionEvent xmotion;
   XKeyEvent xkey;
   XButtonEvent xbutton;
   XConfigureEvent xconfigure;
   XExposeEvent xexpose;
   long pad[24];
} XEvent;

#define ButtonPress     2
#define ButtonRelease   3
#define KeyPress        4
#define KeyRelease      5
#define MotionNotify    6
#define EnterNotify     7
#define LeaveNotify     8
#define Expose          12
#define ConfigureNotify 22


// Proto
#define DefaultColormap(dsp,n)  (0)
#define DefaultVisual(dsp,n)    (0)
#define BlackPixel(dsp,n)       (0)
// fd of X server
#define ConnectionNumber(dsp)   (0)
// HWND of root window
#define RootWindow(dsp,n)       (0)

bool XAllocNamedColor(Display *dsp,Colormap map,string name,XColor *col,XColor* dummy);
void XQueryPointer(Display *dsp,Window w,Window *dummy1,Window *dummy2,int *x,int *y,
                   int *dummy3,int *dummy4,unsigned int *mask);

void XDestroyWindow(Display *dsp,Window w);
void XSync(Display *dsp,Bool discard);

void XFree(void *data);

Status XmuLookupStandardColormap(Display *dsp,int n,int id,
    int depth,int atom,bool replace,bool retain);

Colormap XCreateColormap(Display *dsp,Window w,Visual *visual,int alloc);

// Defines

// Made up
#define DirectColor   1
#define TrueColor     2

#define None          0

// keysym.h
typedef int KeySym;

#define XK_Tab        1

#endif
