/*
 * EGL - Extended OpenGL, codename Eagle
 * NOTES:
 * - Currently a base test for encapsulating the functionality of OpenGL1.2
 * MINIMAL REQUIREMENTS:
 * - True OpenGL1.1 drivers on your system.
 * (c) Ruud van Gaal
 */

#include <egl/egl.h>

static char *sError="EGL Error";
static char *sNotice="EGL Notice";

/*
   Encapsulate a bit of WGL/GLX while we're at it
   To be extended
*/
bool eglMakeCurrent(EGLServer server,EGLDrawable draw,EGLRenderingContext rc)
// Makes a rendering context current to a drawable
{
#ifdef WIN32
  return (bool)wglMakeCurrent(draw,rc);
#endif
#if defined(__sgi)
  glXMakeCurrent(server,draw,rc);
  return true;
#endif
}

void eglSwapBuffers(EGLServer server,EGLDrawable draw)
// Swaps front and back buffer
{
#ifdef WIN32
  SwapBuffers(draw);
#endif
#if defined(__sgi)
  glXSwapBuffers(server,draw);
#endif
}

/*
   Encapsulate OpenGL1.2 calls
*/

// glDrawRangeElements()

void APIENTRY check_glDrawRangeElements(GLenum mode,
  GLuint start,GLuint end,GLsizei count,GLenum type,
  const GLvoid *indices)
{
#ifdef WIN32
  // Find the function
  PFNGLDRAWRANGEELEMENTSWINPROC ptr;
  ptr=(PFNGLDRAWRANGEELEMENTSWINPROC)wglGetProcAddress("glDrawRangeElementsWIN");
  if(ptr)
  {
    glDrawRangeElements=ptr;
    MessageBox(0,"glDrawRangeElements() OpenGL1.2 call WAS FOUND",sNotice,MB_OK);
  } else
  {
    static bool reported;
    if(!reported)
    {
      MessageBox(0,"glDrawRangeElements() OpenGL1.2 call not in driver",sError,MB_OK);
      reported=TRUE;
    }
    return;
  }
  // Call the function
  glDrawRangeElements(mode,start,end,count,type,indices);
#endif
}

ptr_glDrawRangeElements glDrawRangeElements=check_glDrawRangeElements;

/* EXT_blend_color */

// glBlendColor()
void APIENTRY check_glBlendColor(GLclampf red,
  GLclampf green,GLclampf blue,GLclampf alpha)
{
#ifdef WIN32
  // Find the function
  PFNGLBLENDCOLORPROC ptr;
  ptr=(PFNGLBLENDCOLORPROC)wglGetProcAddress("glBlendColorEXT");
  if(ptr)
  {
    glBlendColor=ptr;
    //MessageBox(0,"glBlendColor() call WAS FOUND",sNotice,MB_OK);
  } else
  {
    static bool reported;
    if(!reported)
    {
      MessageBox(0,"glBlendColor() call NOT FOUND",sError,MB_OK);
      reported=TRUE;
    }
    return;
  }
  // Call the function
  glBlendColor(red,green,blue,alpha);
#endif
}

PFNGLBLENDCOLORPROC glBlendColor=check_glBlendColor;

//
// ARB_multitexture
//

void APIENTRY check_glActiveTextureARB(GLenum n)
{
#ifdef WIN32
  // Find the function
  PFNGLACTIVETEXTUREARBPROC ptr;
  ptr=(PFNGLACTIVETEXTUREARBPROC)wglGetProcAddress("glActiveTextureARB");
  if(ptr)
  {
    glActiveTextureARB=ptr;
    //MessageBox(0,"glBlendColor() call WAS FOUND",sNotice,MB_OK);
  } else
  {
    static bool reported;
    if(!reported)
    {
      MessageBox(0,"glActiveTextureARB() call NOT FOUND",sError,MB_OK);
      reported=TRUE;
    }
    return;
  }
  // Call the function
  glActiveTextureARB(n);
#endif
}

PFNGLACTIVETEXTUREARBPROC glActiveTextureARB=check_glActiveTextureARB;
